@@ -11,12 +11,12 @@ class ENCoM(ENM):
 
         It extends the ENM class, implementing its own build_hessian(), pickle() and build_from_pickle() methods. See
         ENM class for the attributes it inherits. For details about the ENCoM model, see the 2014 Frappier and
-        Najmanovich paper: www.doi.org/10.1371/journal.pcbi.1003569
+        Najmanovich paper: `doi.org/10.1371/journal.pcbi.1003569 <https://doi.org/10.1371/journal.pcbi.1003569>`_
 
         Note:
             This class also implements the STeM model (Generalized Spring Tensor Model, on which ENCoM is based). The
             original STeM paper can be found here:
-            `https://doi.org/10.1186/1472-6807-10-S1-S3 <https://doi.org/10.1186/1472-6807-10-S1-S3>`_
+            `doi.org/10.1186/1472-6807-10-S1-S3 <https://doi.org/10.1186/1472-6807-10-S1-S3>`_
 
         Attributes:
             dirpath (str): The absolute path to the directory containing the encom.py module.
@@ -128,6 +128,7 @@ class ENCoM(ENM):
                          use_pickle=use_pickle, solve_mol=solve_mol, one_mass=one_mass)
 
     def _get_pickle_file(self):
+        # TODO : better logic
         pickle_file = self.pdb_file.split('.')[0]
         if self.use_stem:
             pickle_file += ".stem.pickle"
@@ -196,6 +197,59 @@ class ENCoM(ENM):
         V_tot = self._sum_hessians()
         return V_tot
 
+    def build_hybrid_hessian(self, other, solve=True):
+        """Builds the Hessian matrix with V1, V2, V3 from self and V4 from other.
+        """
+        assert isinstance(other, ENCoM)
+        if not self.mol.solved:
+            self.mol.solve()
+        masses = self.mol.masses
+        connect = self.mol.connect
+        bends = self.mol.bends
+        torsions = self.mol.torsions
+        if not other.mol.solved:
+            other.mol.solve()
+        resis = other.mol.resis
+        other_masses = other.mol.masses
+        if self.V1_H is None:
+            self.V1_H = self._build_V1_hessian(masses, connect)
+        if self.V2_H is None:
+            self.V2_H = self._build_V2_hessian(masses, bends)
+        if self.V3_H is None:
+            self.V3_H = self._build_V3_hessian(masses, torsions)
+        if other.V4_H is None:
+            other.V4_H = other._build_V4_hessian(other_masses, resis)
+        self.h = np.add(np.add(np.add(self.V1_H, self.V2_H), self.V3_H), other.V4_H)
+        if solve:
+            self.solve()
+
+    def build_hybrid_hessian_bij(self, other, solve=True):
+        assert isinstance(other, ENCoM)
+        if not self.mol.solved:
+            self.mol.solve()
+        masses = self.mol.masses
+        connect = self.mol.connect
+        bends = self.mol.bends
+        torsions = self.mol.torsions
+        resis = self.mol.resis
+        if not other.mol.solved:
+            other.mol.solve()
+        other_resis = other.mol.resis
+        other_masses = other.mol.masses
+        if self.V1_H is None:
+            self.V1_H = self._build_V1_hessian(masses, connect)
+        if self.V2_H is None:
+            self.V2_H = self._build_V2_hessian(masses, bends)
+        if self.V3_H is None:
+            self.V3_H = self._build_V3_hessian(masses, torsions)
+        if other.bij is None:
+            other.bij = other._compute_Bij(other_masses, other_resis)
+        temp_V4_H = self._build_V4_hessian(masses, resis, bij=other.bij)
+        self.h = np.add(np.add(np.add(self.V1_H, self.V2_H), self.V3_H), temp_V4_H)
+        if solve:
+            self.solve()
+
+
     def _sum_hessians(self):
         return np.add(np.add(np.add(self.V1_H, self.V2_H), self.V3_H), self.V4_H)
 
@@ -378,9 +432,11 @@ class ENCoM(ENM):
             self._hessian_helper_V2V3(hessian, doubles, dGd, const)
         return hessian
 
-    def _build_V4_hessian(self, masses, resis):
+    def _build_V4_hessian(self, masses, resis, bij=None):
         n = len(masses)
-        bij = self._compute_Bij(masses, resis)
+        if bij is None:
+            bij = self._compute_Bij(masses, resis)
+            self.bij = bij
         hessian = np.zeros((3 * n, 3 * n))
         axes = ["X", "Y", "Z"]
         indexes = ["i", "j"]
@@ -400,7 +456,6 @@ class ENCoM(ENM):
                     doubles = [x for x in zip([i, j], indexes)]
                     # self.hessian_helper_V1V4(hessian, doubles, bcoord, const)
                     self._hessian_helper_V1V4_optim(hessian, i, j, bcoord, const)
-        self.bij = bij
         return hessian
 
     def _get_weighted_surf_atomnum(self, sd, num_a, num_b, resi_a, resi_b, anums_dict):
@@ -495,3 +550,7 @@ class ENCoM(ENM):
 
 def _distance(a, b):
     return ((a[0] - b[0]) ** 2 + (a[1] - b[1]) ** 2 + (a[2] - b[2]) ** 2) ** 0.5
+
+if __name__ == "__main__":
+    test = ENCoM("/Users/Paracelsus/school/PhD/h21/projet/encom_rna/md_analysis/trajectories/mir125_AA_1_model1.pdb")
+    print(test.compute_bfactors())
