@@ -172,7 +172,7 @@ class ENM(metaclass=abc.ABCMeta):
                     return False
         return True
 
-    def compute_bfactors(self, filter=None):
+    def compute_bfactors(self, filter=None, modes_proportion=1):
         """Computes the predicted b-factors.
 
         The root-mean-square flutuations of every mass can be seen as a prediction of the experimental temperature
@@ -193,25 +193,18 @@ class ENM(metaclass=abc.ABCMeta):
         assert self.eigvecs is not None
         n = int(len(self.eigvecs) / 3)
         bfacts = []
+        end_j = int((n * 3 - 6) * modes_proportion) + 6
         for i in range(n):
             bfact = 0
-            for j in range(6, n * 3):  # skips 1st 6 rotation-translation motions
+            for j in range(6, end_j):  # skips 1st 6 rotation-translation motions
                 temp = 0
                 for k in range(3):
                     temp += self.eigvecs[j][3 * i + k] ** 2
                 bfact += temp / self.eigvals[j]
             bfacts.append(bfact * 1000)
-        self.bfacts = bfacts
-        if filter is not None:
-            kept_bfacts = []
-            masses = self.mol.masses
-            for i, bfact in enumerate(self.bfacts):
-                resiname = masses[i][4]
-                if resiname.split('|')[0].split('.')[-1] in filter:
-                    kept_bfacts.append(bfact)
-            return kept_bfacts
-        else:
-            return self.bfacts
+        filtered_bfacts = self.filter_bfactors(bfacts, filter)
+        self.bfacts = filtered_bfacts
+        return filtered_bfacts
 
     def get_exp_bfacts(self, method="average", filter=None):
         """Extracts the experimental b-factors for every mass in the system.
@@ -258,6 +251,46 @@ class ENM(metaclass=abc.ABCMeta):
         self.entropy = ent
         return ent
 
+    def filter_bfactors(self, bfacts, filter):
+        if filter is not None:
+            kept_bfacts = []
+            masses = self.mol.masses
+            for i, bfact in enumerate(self.bfacts):
+                resiname = masses[i][4]
+                if resiname.split('|')[0].split('.')[-1] in filter:
+                    kept_bfacts.append(bfact)
+            return kept_bfacts
+        else:
+            return bfacts
+
+    def compute_bfactors_boltzmann(self, beta=None, factor=1, filter=None, modes_proportion=1):
+        assert self.eigvals is not None
+        assert self.eigvecs is not None
+        n = int(len(self.eigvecs) / 3)
+        bfacts = []
+        end_j = int((n * 3 - 6) * modes_proportion) + 6
+        if beta is None:
+            beta = 1.55 * 10 ** -13  # beta is h/kT, this value is for T = 310 K
+        beta *= factor
+        e = 2.718281828459045
+        pi = 3.1415926535897932384626433832795028841971693993751
+        entros = [None, None, None, None, None, None]
+        for j in range(6, end_j):
+            vj = (self.eigvals[j] ** 0.5) / (2 * pi)
+            x = vj * beta
+            entros.append(x / (e ** x - 1) - np.log(1 - e ** (-1 * x)))
+        for i in range(n):
+            bfact = 0
+            for j in range(6, end_j):  # skips 1st 6 rotation-translation motions
+                temp = 0
+                for k in range(3):
+                    temp += self.eigvecs[j][3 * i + k] ** 2
+                bfact += temp * entros[j]
+            bfacts.append(bfact * 1000)
+        filtered_bfacts = self.filter_bfactors(bfacts, filter)
+        self.bfacts = filtered_bfacts
+        return filtered_bfacts
+
     def compute_vib_entropy(self, beta=None, factor=1):
         """Vibrational entropy from the eigenfrequencies (without rigid-rotor approximation).
 
@@ -578,22 +611,33 @@ class ENM(metaclass=abc.ABCMeta):
         assert self.get_n_masses() == len(vector)
         self.mol.set_bfactors(vector)
 
-    def write_dynamical_signature(self, outfile):
+    def write_dynamical_signature(self, outfile, normalize=False):
         """Writes a 'dynamical signature' to the output file specified.
 
         The dynamical signature is simply the predicted b_factor at every mass.
 
         Args:
-            outfile: the output file.
+            outfile (str): the output file.
+            normalize (bool, optional): if True, the signature is normalized to 1
         """
-        if not hasattr(self, "bfacts"):
+        if not hasattr(self, "bfacts") or self.bfacts is None:
             self.compute_bfactors()
-        masses = self.mol.masses
+        labels = self.get_mass_labels()
         bfacts = self.bfacts
-        assert len(masses) == len(bfacts)
+        divider = 1
+        if normalize:
+            divider = np.sum(bfacts)
+        assert len(labels) == len(bfacts)
         with open(outfile, "w") as f:
-            for i in range(len(masses)):
-                f.write("{0}\t{1}\n".format(masses[i][4], bfacts[i]))
+            for label, bfact in zip(labels, bfacts):
+                f.write("{0}\t{1}\n".format(label, bfact / divider))
+
+    def get_mass_labels(self):
+        masses = self.mol.masses
+        labels = []
+        for mass in masses:
+            labels.append(mass[4])
+        return labels
 
     def _write_to_file(self, filename):
         self.mol.write_to_file(filename)
@@ -637,6 +681,92 @@ class ENM(metaclass=abc.ABCMeta):
         return np.array(basis)
 
 
+# End of ENM class, start of utility methods ###########################################################################
+
+
+def generate_dynasigs_df(filenames, outname, id_func=None, beta_values=None, models=None, models_labels=None,
+                         additional_info_dict=None, add_info_labels=None,
+                         added_atypes_list=None, added_massdef_list=None, get_cf_from_file=False):
+    if models is None:
+        from nrgten.encom import ENCoM
+        models = [ENCoM]
+        models_labels = ["ENCoM"]
+    if beta_values is None:
+        beta_values = [1]
+    if id_func is None:
+        id_func = lambda x: x.split('/')[-1]
+    assert isinstance(beta_values, list) and isinstance(models, list) and isinstance(models_labels, list)
+    assert len(models) == len(models_labels)
+    if added_atypes_list is not None:
+        assert len(added_atypes_list) == len(filenames) == len(added_massdef_list)
+    dynasigs_data = []
+    dynasigs_masslabels = None
+    ids_data = []
+    betas_data = []
+    vib_entro_data = []
+    mod_labels_data = []
+    cfs = []
+    for i, filename in enumerate(filenames):
+        filename_id = id_func(filename)
+        for mod, mod_lab in zip(models, models_labels):
+            if added_atypes_list is None:
+                enm = mod(filename)
+            else:
+                enm = mod(filename, added_atypes=added_atypes_list[i], added_massdef=added_massdef_list[i])
+            masslabels = [x.split('.')[-1] for x in enm.get_mass_labels()]  # only the mass name to allow for mutations
+            for beta_val in beta_values:
+                dynasigs_data.append(enm.compute_bfactors_boltzmann(beta=beta_val))
+                vib_entro_data.append(enm.compute_vib_entropy(beta=beta_val))
+                if dynasigs_masslabels is None:
+                    dynasigs_masslabels = masslabels
+                else:
+                    validate_masslabels(masslabels, dynasigs_masslabels)
+                ids_data.append(filename_id)
+                betas_data.append(beta_val)
+                mod_labels_data.append(mod_lab)
+                if get_cf_from_file:
+                    cf = None
+                    with open(filename) as f:
+                        lines = f.readlines()
+                    for line in lines:
+                        if line.startswith("REMARK CF="):
+                            cf = float(line[10:])
+                            break
+                    if cf is not None:
+                        cfs.append(cf)
+                    else:
+                        raise ValueError("Could not find CF line in file {}".format(filename))
+    colnames = ["id", "model", "beta", "vib_entro"]
+    if get_cf_from_file:
+        colnames.append("cf")
+    if additional_info_dict is not None:
+        assert add_info_labels is not None and isinstance(add_info_labels, list)
+        colnames = colnames + add_info_labels
+    sig_len = len(dynasigs_masslabels)
+    colnames = colnames + dynasigs_masslabels
+    df_strs = [" ".join(colnames)]
+    for i in range(len(dynasigs_data)):
+        assert len(dynasigs_data[i]) == sig_len
+        current_str = "{} {} {} {}".format(ids_data[i], mod_labels_data[i], betas_data[i], vib_entro_data[i])
+        if get_cf_from_file:
+            current_str = current_str + " {}".format(cfs[i])
+        if additional_info_dict is not None:
+            current_str = current_str + " " + " ".join(
+                [additional_info_dict[ids_data[i]][label] for label in add_info_labels])
+        current_str = current_str + " " + " ".join([str(x) for x in dynasigs_data[i]])
+        df_strs.append(current_str)
+    with open(outname, "w") as f:
+        f.write("\n".join(df_strs))
+        f.write("\n")
+
+
+def validate_masslabels(new_labels, old_labels):
+    assert len(new_labels) == len(old_labels)
+    for new, old in zip(new_labels, old_labels):
+        if new != old:
+            raise ValueError("problem with masslabels in generate_dynasigs_df: {} not equal to {}".format(new, old))
+
+
 
 
 
